﻿using System;
using System.Runtime.InteropServices;

namespace RefApp8
{
    public static class DeviceNotification
    {
        /// <summary>
        /// ウィンドウが通知を受信するデバイスまたはデバイスの種類を登録します
        /// </summary>
        /// <param name="hRecipient">NotificationFilter パラメーターで指定されたデバイスのデバイス イベントを受信するウィンドウまたはサービスへのハンドル</param>
        /// <param name="NotificationFilter">通知を送信するデバイスの種類を指定するデータ ブロックへのポインター</param>
        /// <param name="Flags"></param>
        /// <returns>デバイス通知ハンドル</returns>
        /// <remarks>
        /// https://learn.microsoft.com/ja-jp/windows/win32/devio/registering-for-device-notification <br />
        /// https://learn.microsoft.com/ja-jp/windows/win32/api/winuser/nf-winuser-registerdevicenotificationw?source=recommendations
        /// </remarks>
        [DllImport("user32.dll", SetLastError = true)]
        public static extern IntPtr RegisterDeviceNotification(IntPtr hRecipient, IntPtr NotificationFilter, uint Flags);

        /// <summary>
        /// hRecipient パラメーターは、サービス状態ハンドルです
        /// </summary>
        public const int DEVICE_NOTIFY_SERVICE_HANDLE = 0x00000001;

        /// <summary>
        /// hRecipient パラメーターはウィンドウ ハンドルです
        /// </summary>
        public const int DEVICE_NOTIFY_WINDOW_HANDLE = 0x00000000;

        /// <summary>
        /// すべてのデバイスインターフェイスクラスのデバイスインターフェイスイベントを受信者に通知します。
        /// (dbcc_classguid メンバーは無視されます。)
        /// この値は dbch_devicetype メンバーが DBT_DEVTYP_DEVICEINTERFACE に設定されている場合にのみ使用できます。
        /// </summary>
        /// <remarks>
        /// https://learn.microsoft.com/ja-jp/windows/win32/api/winuser/nf-winuser-registerdevicenotificationa
        /// </remarks>
        public const uint DEVICE_NOTIFY_ALL_INTERFACE_CLASSES = 0x00000004;

        /// <summary>
        /// デバイスのクラスに関する情報が含まれます
        /// </summary>
        /// <remarks>
        /// https://learn.microsoft.com/ja-jp/windows/win32/api/dbt/ns-dbt-dev_broadcast_deviceinterface_a
        /// </remarks>
        [StructLayout(LayoutKind.Sequential)]
        public class DEV_BROADCAST_DEVICEINTERFACE
        {
            public uint dbcc_size;
            public uint dbcc_devicetype;
            public uint dbcc_reserved;
            public Guid dbcc_classguid;
            public char dbcc_name;
        };

        /// <summary>
        /// デバイスインターフェースクラス
        /// </summary>
        public const uint DBT_DEVTYP_DEVICEINTERFACE = 0x00000005;

        /// <summary>
        /// USB挿抜監視の登録を行います
        /// </summary>
        /// <param name="hRecipient">通知先ウィンドウハンドル</param>
        /// <returns>正常時true</returns>
        public static bool Register(IntPtr hRecipient)
        {
            var NotificationFilterSize = Marshal.SizeOf(typeof(DEV_BROADCAST_DEVICEINTERFACE));

            var NotificationFilter = new DEV_BROADCAST_DEVICEINTERFACE
            {
                dbcc_size = (uint)NotificationFilterSize,
                dbcc_devicetype = DBT_DEVTYP_DEVICEINTERFACE,
                dbcc_reserved = 0,
            };

            var NotificationFilterBuffer = Marshal.AllocHGlobal(NotificationFilterSize);
            Marshal.StructureToPtr(NotificationFilter, NotificationFilterBuffer, true);

            var Flags = DEVICE_NOTIFY_ALL_INTERFACE_CLASSES;
            var hDeviceNotify = RegisterDeviceNotification(hRecipient, NotificationFilterBuffer, Flags);

            return hDeviceNotify != IntPtr.Zero;
        }

        /// <summary>
        /// WM_DEVICECHANGE メッセージを介して報告されるデバイス イベントに関連する情報の標準ヘッダーとして機能します
        /// </summary>
        /// <remarks>
        /// https://learn.microsoft.com/ja-jp/windows/win32/api/dbt/ns-dbt-dev_broadcast_hdr
        /// </remarks>
        [StructLayout(LayoutKind.Sequential)]
        public class DEV_BROADCAST_HDR
        {
            public uint dbch_size;
            public uint dbch_devicetype;
            public uint dbch_reserved;
        }

        /// <summary>
        /// デバイスまたはコンピューターのハードウェア構成の変更をアプリケーションに通知します
        /// </summary>
        public const int WM_DEVICECHANGE = 0x0219;

        /// <summary>
        /// WM_DEVICECHANGEメッセージで通知されるデバイスのイベントの種類です
        /// </summary>
        public enum DBT : long
        {
            /// <summary>システムは、デバイスまたはメディアの一部が挿入されて使用可能になったときに、DBT_DEVICEARRIVAL デバイス イベントをブロードキャストします</summary>
            DBT_DEVICEARRIVAL = 0x8000,

            /// <summary>システムは、DBT_DEVICEQUERYREMOVE デバイス イベントをブロードキャストして、デバイスまたはメディアの一部を削除するアクセス許可を要求します</summary>
            DBT_DEVICEQUERYREMOVE = 0x8001,

            /// <summary>デバイスまたはメディアの削除要求が取り消されたときに、DBT_DEVICEQUERYREMOVEFAILED デバイス イベントがブロードキャストされます</summary>
            DBT_DEVICEQUERYREMOVEFAILED = 0x8002,

            /// <summary>システムは、デバイスまたはメディアの一部が削除され、使用できなくなったときに、DBT_DEVICEREMOVEPENDING デバイス イベントをブロードキャストします</summary>
            DBT_DEVICEREMOVEPENDING = 0x8003,

            /// <summary>システムは、デバイスまたはメディアの一部が物理的に削除されたときに、DBT_DEVICEREMOVECOMPLETEデバイス イベントをブロードキャストします</summary>
            DBT_DEVICEREMOVECOMPLETE = 0x8004,

            /// <summary>デバイスがシステムに追加またはシステムから削除されたときに、DBT_DEVNODES_CHANGED デバイス イベントをブロードキャストします</summary>
            DBT_DEVNODES_CHANGED = 0x0007,
        }

        /// <summary>
        /// ウィンドウプロシージャです
        /// </summary>
        /// <param name="_">メッセージ通知先ウィンドウハンドルです(使用しません)</param>
        /// <param name="msg">ウィンドウメッセージです</param>
        /// <param name="wParam">wParamです</param>
        /// <param name="lParam">lParamです</param>
        /// <param name="handled">ウィンドウプロシージャでウィンドウメッセージを処理した場合はtrueを設定します</param>
        /// <returns>常にIntPtr.Zeroを返します</returns>
        public static IntPtr WindowProcedure(IntPtr _, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            if (msg == WM_DEVICECHANGE)
            {
                if (lParam == IntPtr.Zero)
                {
                    return IntPtr.Zero;
                }

                var devHdr = new DEV_BROADCAST_HDR();
                Marshal.PtrToStructure(lParam, devHdr);
                if (devHdr.dbch_devicetype != DBT_DEVTYP_DEVICEINTERFACE)
                {
                    return IntPtr.Zero;
                }

                if (wParam.ToInt64() == (long)DBT.DBT_DEVICEARRIVAL)
                {
                    OnDeviceArrival(lParam);
                }
                if (wParam.ToInt64() == (long)DBT.DBT_DEVICEREMOVECOMPLETE)
                {
                    OnDeviceRemoveComplete(lParam);
                }
            }

            handled = false;
            return IntPtr.Zero;
        }

        /// <summary>
        /// USBデバイス切断時の処理です
        /// </summary>
        /// <param name="lParam">lParamです</param>
        private static void OnDeviceRemoveComplete(IntPtr lParam)
        {
            var devInterface = new DEV_BROADCAST_DEVICEINTERFACE();
            Marshal.PtrToStructure(lParam, devInterface);
            if (!devInterface.dbcc_classguid.Equals(SUS.DeviceGUID))
            {
                return;
            }

            if (!SUS.UsbClose())
            {
                MainWindow.LogView.Print("ボードとの切断に失敗しました。");
            }
            else
            {
                MainWindow.LogView.Print("ボードとの切断に成功しました。");
            }
            MainWindow.Instance.TabPagesEnabled = false;
        }

        /// <summary>
        /// USBデバイス接続時の処理です
        /// </summary>
        /// <param name="lParam">lParamです</param>
        private static void OnDeviceArrival(IntPtr lParam)
        {
            var devInterface = new DEV_BROADCAST_DEVICEINTERFACE();
            Marshal.PtrToStructure(lParam, devInterface);
            if (!devInterface.dbcc_classguid.Equals(SUS.DeviceGUID))
            {
                return;
            }

            if (!SUS.UsbOpen())
            {
                MainWindow.LogView.Print("ボードとの接続に失敗しました。");
                MainWindow.Instance.TabPagesEnabled = false;
            }
            else
            {
                MainWindow.LogView.Print("ボードとの接続に成功しました。");
                MainWindow.Instance.TabPagesEnabled = true;
            }
        }
    }
}
