﻿using RefApp8.Util;
using System;
using System.Diagnostics;
using System.IO;
using System.Windows;
using System.Windows.Interop;

namespace RefApp8
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// ログウィンドウインスタンス
        /// </summary>
        /// <remarks>
        /// 同一プロセスでアプリの画面は複数生成しない前提です
        /// </remarks>
        static private LogWindow logWindow = null;

        /// <summary>
        /// ログウィンドウアクセサ
        /// </summary>
        /// <remarks>
        /// 同一プロセスでアプリの画面は複数生成しない前提です
        /// </remarks>
        static public LogWindow LogView
        {
            get 
            {
                if (logWindow == null)
                {
                    logWindow = new LogWindow();
                }
                return logWindow;
            }
            set { logWindow = value; }
        }

        /// <summary>
        /// メインウィンドウインスタンスです
        /// </summary>
        /// <remarks>
        /// 同一プロセスでアプリの画面は複数生成しない前提です
        /// </remarks>
        static public MainWindow Instance { get; set; }

        /// <summary>
        /// ログファイルのフルパスです
        /// </summary>
        static public string LogPath { get; private set; }

        /// <summary>
        /// エラーメッセージダイアログのキャプションです
        /// </summary>
        public const string ErrorCaption = "RefApp8 エラー";

        /// <summary>
        /// 3つのタブページの活性状態を設定します
        /// </summary>
        public bool TabPagesEnabled
        {
            get
            {
                return TabPageRegister.IsEnabled;
            }
            set
            {
                TabPageRegister.IsEnabled = TabPageMemory.IsEnabled = TabPageConfig.IsEnabled = value;
            }
        }

        /// <summary>
        /// SUSカードが接続されているか示します
        /// </summary>
        public bool CardAttached { get; private set; }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public MainWindow()
        {
            // アプリケーション設定ファイルの読み込み
            Util.Config.Open();

            // 画面コンポーネントの初期化
            InitializeComponent();
            Instance = this;

            // ログファイル名を作成
            var logFile = DateTime.Now.ToString("yyyymmdd-HHMMss") + ".log";
            LogPath = Path.Combine(AppData.LogDirectory, logFile);

            // ログ出力画面の表示
            LogView.Show();

            // WinUSBドライババージョンの取得
            GetWinUsbDriverVersion();

            // アプリ起動時にSUSデバイスのオープン処理を行う
            LogView.Print("SUSデバイスをオープンしています。。。");
            CardAttached = SUS.UsbOpen();
            TabPagesEnabled = CardAttached;
        }

        /// <summary>
        /// メインウィンドウを読み込んだ時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">キャンセルイベント引数</param>
        private void Window_Loaded(object sender, EventArgs e)
        {
            // メモリ読み出し・書き込みデータファイルのパスを入力欄に反映
            TabPageMemory.FilePath.Text = Util.Config.MemoryDataPath;

            // FPGAコンフィグファイルのパスを入力欄に反映
            TabPageConfig.FilePath.Text = Util.Config.ConfigDataPath;
            // FPGAチップベンダー種別を入力欄に反映
            (Util.Config.IsVenderXilinx ? TabPageConfig.VenderXilinx : TabPageConfig.VenderAltera).IsChecked = true;

            // トップレベルウィンドウハンドルの取得
            var process = Process.GetCurrentProcess();
            var windowHandle = process.MainWindowHandle;

            // USBイベント通知設定
            if (!DeviceNotification.Register(windowHandle))
            {
                LogView.Print("USBイベント通知設定に失敗しました。");
            }
            else
            {
                LogView.Print("USBイベント通知設定に成功しました。");
            }

            // ステータスバー設定
            if (!CardAttached)
            {
                LogView.Print("SUSデバイスのオープンに失敗しました。");
                return;
            }
            else
            {
                LogView.Print("SUSデバイスをオープンしました。");
            }
        }

        /// <summary>
        /// メインウィンドウを閉じるときのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">キャンセルイベント引数</param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            LogView.Close();
        }

        /// <summary>
        /// 終了ボタンクリック時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void Exit_Click(object sender, RoutedEventArgs e)
        {
            SUS.UsbClose();
            LogView.Close();
            Close();
        }

        /// <summary>
        /// winusb.sysのドライババージョンを取得します
        /// </summary>
        private void GetWinUsbDriverVersion()
        {
            try
            {
                var path = Environment.SystemDirectory + @"\drivers\winusb.sys";
                var VerInfo = FileVersionInfo.GetVersionInfo(path);
                LogView.Print($"winusb.sys = Ver.{VerInfo.FileVersion}");
            }
            catch (Exception ex)
            {
                LogView.Print($"winusb.sysドライバのバージョン取得に失敗しました: {ex.Message}");
            }
        }

        /// <summary>
        /// ログ再表示ボタンのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元オブジェクト</param>
        /// <param name="e">イベント引数</param>
        private void ShowLog_Click(object sender, RoutedEventArgs e)
        {
            LogView.Show();
        }

        /// <summary>
        /// ログファイルを開くボタンのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元オブジェクト</param>
        /// <param name="e">イベント引数</param>
        private void OpenLog_Click(object sender, RoutedEventArgs e)
        {
            Process.Start(LogPath);
        }

        /// <summary>
        /// ウィンドウプロシージャ相当のメソッドを設定します
        /// </summary>
        /// <param name="e">イベント引数</param>
        protected override void OnSourceInitialized(EventArgs e)
        {
            base.OnSourceInitialized(e);
            var windowHelper = new WindowInteropHelper(this);
            var windowHandle = windowHelper.Handle;
            var source = HwndSource.FromHwnd(windowHandle);
            source.AddHook(new HwndSourceHook(DeviceNotification.WindowProcedure));
        }
    }
}
