﻿using System;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace RefApp8
{
    /// <summary>
    /// NumericUpDown.xaml の相互作用ロジック
    /// </summary>
    public partial class NumericUpDown : UserControl
    {
        public static readonly DependencyProperty ValueProperty = DependencyProperty.Register("Value", typeof(int), typeof(NumericUpDown), new PropertyMetadata(0));
        public static readonly DependencyProperty MinValueProperty = DependencyProperty.Register("MinValue", typeof(int), typeof(NumericUpDown), new PropertyMetadata(0));
        public static readonly DependencyProperty MaxValueProperty = DependencyProperty.Register("MaxValue", typeof(int), typeof(NumericUpDown), new PropertyMetadata(0));

        /// <summary>
        /// NumericUpDownコントロールが保持する値
        /// </summary>
        public int Value
        {
            get { return (int)GetValue(ValueProperty); }
            set { SetValue(ValueProperty, value); }
        }

        /// <summary>
        /// NumericUpDownコントロールが値域の下限値
        /// </summary>
        public int MinValue
        {
            get { return (int)GetValue(MinValueProperty); }
            set { SetValue(MinValueProperty, value); }
        }

        /// <summary>
        /// NumericUpDownコントロールが値域の上限値
        /// </summary>
        public int MaxValue
        {
            get { return (int)GetValue(MaxValueProperty); }
            set { SetValue(MaxValueProperty, value); }
        }

        /// <summary>
        /// コンストラクタです
        /// </summary>
        public NumericUpDown()
        {
            InitializeComponent();
            Value = 0;
            ValueText.Text = Value.ToString();
        }

        /// <summary>
        /// [↑]ボタンクリック時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ValueUp_Click(object sender, RoutedEventArgs e)
        {
            if (ValueText.Text != "")
            {
                Value = Convert.ToInt32(ValueText.Text);
            }
            else
            {
                Value = 0;
            }
            if (Value < MaxValue)
            {
                ++Value;
                ValueText.Text = Convert.ToString(Value);
            }
        }

        /// <summary>
        /// [↓]ボタンクリック時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ValueDown_Click(object sender, RoutedEventArgs e)
        {
            if (ValueText.Text != "")
            {
                Value = Convert.ToInt32(ValueText.Text);
            }
            else
            {
                Value = 0;
            }
            if (Value > MinValue)
            {
                --Value;
                ValueText.Text = Convert.ToString(Value);
            }
        }

        /// <summary>
        /// テキストボックスのキー押下時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ValueText_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Up)
            {
                ValueUp.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                typeof(Button).GetMethod("set_IsPressed", BindingFlags.Instance | BindingFlags.NonPublic).Invoke(ValueUp, new object[] { true });
            }

            if (e.Key == Key.Down)
            {
                ValueDown.RaiseEvent(new RoutedEventArgs(Button.ClickEvent));
                typeof(Button).GetMethod("set_IsPressed", BindingFlags.Instance | BindingFlags.NonPublic).Invoke(ValueDown, new object[] { true });
            }
        }

        /// <summary>
        /// テキストボックスのキー押下後のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ValueText_PreviewKeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Up)
                typeof(Button).GetMethod("set_IsPressed", BindingFlags.Instance | BindingFlags.NonPublic).Invoke(ValueUp, new object[] { false });

            if (e.Key == Key.Down)
                typeof(Button).GetMethod("set_IsPressed", BindingFlags.Instance | BindingFlags.NonPublic).Invoke(ValueDown, new object[] { false });
        }

        /// <summary>
        /// テキストボックスのテキスト編集後のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void Value_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (ValueText.Text != "")
            {
                if (int.TryParse(ValueText.Text, out int value))
                {
                    Value = value;
                    ValueText.Text = Value.ToString();
                }
            }
            if (Value > MaxValue)
            {
                ValueText.Text = MaxValue.ToString();
            }
            if (Value < MinValue)
            {
                ValueText.Text = MinValue.ToString();
            }
            ValueText.SelectionStart = ValueText.Text.Length;
        }
    }
}
