﻿using System;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;

namespace RefApp8
{
    /// <summary>
    /// SUSv.dllのインターフェースを提供するクラスです
    /// </summary>
    public static class SUS
    {
        /// <summary>
        /// PSIカードのクラス定義です
        /// </summary>
        /// <remarks>
        /// [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\DeviceClasses\{82efb874-53f7-4595-9b80-5388380a1b3a}]
        /// 
        /// [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\DeviceClasses\{82efb874-53f7-4595-9b80-5388380a1b3a}\##?#USB#VID_0A2F&PID_8003#6&3af0f9ce&0&13#{82efb874-53f7-4595-9b80-5388380a1b3a}]
        /// "DeviceInstance"="USB\\VID_0A2F&PID_8003\\6&3af0f9ce&0&13"
        /// 
        /// [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\DeviceClasses\{82efb874-53f7-4595-9b80-5388380a1b3a}\##?#USB#VID_0A2F&PID_8003#6&3af0f9ce&0&13#{82efb874-53f7-4595-9b80-5388380a1b3a}\#]
        /// 
        /// [HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Control\DeviceClasses\{82efb874-53f7-4595-9b80-5388380a1b3a}\##?#USB#VID_0A2F&PID_8003#6&3af0f9ce&0&4#{82efb874-53f7-4595-9b80-5388380a1b3a}]
        /// "DeviceInstance"="USB\\VID_0A2F&PID_8003\\6&3af0f9ce&0&4"
        /// </remarks>
        public const uint VID = 0x0A2F;
        public const uint PID = 0x8003;
        public static readonly string DeviceID = $"VID_{VID:X4}&PID_{PID:X4}";
        public static readonly Guid DeviceGUID = new Guid("{82efb874-53f7-4595-9b80-5388380a1b3a}");

        /// <summary>
        /// オープンした USB デバイスのデバイスハンドルを返します
        /// </summary>
        public static IntPtr FoundHandle => dFoundHadle;

        /// <summary>
        /// オープンした USB デバイスの WinUSB ハンドルを返します
        /// </summary>
        public static IntPtr WinUsbHandle => dWinUsbHandle;

        /// <summary>
        /// オープンした USB デバイスのデバイスハンドル
        /// </summary>
        private static IntPtr dFoundHadle = IntPtr.Zero;

        /// <summary>
        /// オープンした USB デバイスの WinUSB ハンドル
        /// </summary>
        private static IntPtr dWinUsbHandle = IntPtr.Zero;

        /// <summary>
        /// 最初に見つかった 1 台の USB デバイスをオープンします 
        /// </summary>
        /// <param name="dFoundHandle">オープンした USB デバイスのデバイスハンドルを格納します</param>
        /// <param name="dWinUsbHandle">オープンした USB デバイスの WinUSB ハンドルを格納します</param>
        /// <returns>USB デバイスを Open できた場合  TRUE を返します。それ以外は FALSE を返します。</returns>
        /// <remarks>
        /// 対応する USB デバイスが接続されていない場合やすでにオープンされている場合は失敗します。 
        /// </remarks>
        [DllImport("SUSlv.dll")]
        public static extern bool SUSlv_Usb_Open(ref IntPtr dFoundHandle, ref IntPtr dWinUsbHandle);

        /// <summary>
        /// 最初に見つかった 1 台の USB デバイスをオープンします 
        /// </summary>
        /// <returns>USB デバイスを Open できた場合  TRUE を返します。それ以外は FALSE を返します。</returns>
        /// <remarks>
        /// 対応する USB デバイスが接続されていない場合やすでにオープンされている場合は失敗します。 <br />
        /// DLLが提供するAPIのアクセスに必要なハンドルは内部に保持します。
        /// </remarks>
        public static bool UsbOpen()
        {
            return SUSlv_Usb_Open(ref dFoundHadle, ref dWinUsbHandle);
        }

        /// <summary>
        /// USB デバイスをクローズします
        /// </summary>
        /// <param name="dFoundHandle">オープンしたときに取得した USB デバイスのデバイスハンドルを指定します</param>
        /// <param name="dWinUsbHandle">クローズする USB デバイスの WinUSB ハンドルを指定します</param>
        /// <returns> Close が成功した場合  TRUE を返します。それ以外は FALSE を返します。</returns>
        /// <remarks>
        /// オープンしている USB デバイスが抜かれた時にはそのタイミングで必ずクローズを行って下さい。 
        /// </remarks>
        [DllImport("SUSlv.dll")]
        public static extern bool SUSlv_Usb_Close(IntPtr dFoundHandle, IntPtr dWinUsbHandle);

        /// <summary>
        /// USB デバイスをクローズします
        /// </summary>
        /// <returns> Close が成功した場合  TRUE を返します。それ以外は FALSE を返します。</returns>
        /// <remarks>
        /// オープンしている USB デバイスが抜かれた時にはそのタイミングで必ずクローズを行って下さい。<br />
        /// DLLが提供するAPIのアクセスに必要なハンドルは内部に保持した値を使用します。 
        /// </remarks>
        public static bool UsbClose()
        {
            if (SUSlv_Usb_Close(dFoundHadle, dWinUsbHandle))
            {
                dFoundHadle = IntPtr.Zero;
                dWinUsbHandle = IntPtr.Zero;
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// 任意のレジスタから値を読み込みます
        /// </summary>
        /// <param name="dWinUSBHadle">動作を実行する USB デバイスの WinUSB ハンドルを指定します</param>
        /// <param name="address">対象となるレジスタ番号を指定します。0~2,097,151まで設定できます。</param>
        /// <param name="size">バイト幅を指定します。1から512バイトまでの範囲です。</param>
        /// <param name="read_bytes">読み込んだ値を受け取るバッファへのポインタを指定します。</param>
        /// <returns>レジスタ読み込みが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        [DllImport("SUSlv.dll")]
        public static extern bool SUSlv_Reg_Read(IntPtr dWinUSBHadle, uint address, uint size, byte[] read_bytes);

        /// <summary>
        /// レジスタビット長を表現する列挙値です
        /// </summary>
        public enum RegisterWidth
        {
            UNSPECIFIED = -1,
            RANGE_8BIT,
            RANGE_16BIT,
            RANGE_32BIT,
            RANGE_64BIT,
        }

        /// <summary>
        /// 任意のレジスタから値を読み込みます
        /// </summary>
        /// <param name="address">対象となるレジスタ番号を指定します。0~2,097,151まで設定できます。</param>
        /// <param name="registerWidth">レジスタビット長を表現する列挙値です</param>
        /// <param name="readData">読み込んだ値を16進数文字列に変換した結果を格納する文字列変数を指定します</param>
        /// <param name="message">ステータスメッセージを返します</param>
        /// <returns>レジスタ読み込みが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        public static bool RegRead(uint address, RegisterWidth registerWidth, out string readData, out string message)
        {
            // レジスタ読込結果の16進数文字列変数を空で初期化
            readData = "";
            // エラーメッセージ文字列変数を空で初期化
            message = "";

            // 有効なハンドルが取得できていないときはエラー
            if (WinUsbHandle == IntPtr.Zero)
            {
                message = "USBオープンされていません。";
                return false;
            }

            // レジスタビット長が未指定
            if (registerWidth == RegisterWidth.UNSPECIFIED)
            {
                message = "レジスタビット長を指定してください。";
                return false;
            }

            // レジスタビット長の取得
            var size = (1 << (int)registerWidth);

            // レジスタリード処理のDLL関数コール
            var readBytes = new byte[size];
            if (!SUSlv_Reg_Read(WinUsbHandle, address, (uint)size, readBytes))
            {
                message = $"アドレス 0x{address:X8} の{size * 8}ビットレジスタの読み込みに失敗しました。";
                return false;
            }

            // 出力する16進数文字列の組み立て
            foreach (var i in Enumerable.Range(0, size))
            {
                readData += $"{readBytes[size - i - 1]:X2}";
            }

            message = $"アドレス 0x{address:X8} の{size * 8}ビットレジスタの読み込みに成功しました: {readData}";
            return true;
        }

        /// <summary>
        /// 任意のレジスタから値を書き込みます
        /// </summary>
        /// <param name="dWinUSBHadle">動作を実行する USB デバイスの WinUSB ハンドルを指定します</param>
        /// <param name="address">対象となるレジスタ番号を指定します。0~2,097,151まで設定できます。</param>
        /// <param name="size">バイト幅を指定します。1から512バイトまでの範囲です。</param>
        /// <param name="read_bytes">書き込むデータバッファへのポインタを指定します。</param>
        /// <returns>レジスタ書き込みが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        [DllImport("SUSlv.dll")]
        public static extern bool SUSlv_Reg_Write(IntPtr dWinUSBHadle, uint address, uint size, byte[] read_bytes);

        /// <summary>
        /// 任意のレジスタから値を読み込みます
        /// </summary>
        /// <param name="address">対象となるレジスタ番号を指定します。0~2,097,151まで設定できます。</param>
        /// <param name="registerWidth">レジスタビット長を表現する列挙値です</param>
        /// <param name="writeData">読み込んだ値を16進数文字列に変換した結果を格納する文字列変数を指定します</param>
        /// <param name="message">ステータスメッセージを返します</param>
        /// <returns>レジスタ読み込みが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        public static bool RegWrite(uint address, RegisterWidth registerWidth, string writeData, out string message)
        {
            // エラーメッセージ文字列変数を空で初期化
            message = "";

            // 有効なハンドルが取得できていないときはエラー
            if (WinUsbHandle == IntPtr.Zero)
            {
                message = "USBオープンされていません。";
                return false;
            }

            // レジスタビット長が未指定
            if (registerWidth == RegisterWidth.UNSPECIFIED)
            {
                message = "レジスタビット長を指定してください。";
                return false;
            }

            // レジスタビット長の取得
            var size = 1 << (int)registerWidth;

            // 書き込みデータを指定桁数にパディング
            writeData = writeData.PadLeft(size * 2, '0');

            // 書き込みバイト列の生成
            var writeBytes = new byte[size];
            foreach (var i in Enumerable.Range(0, size))
            {
                if (!byte.TryParse(writeData.Substring(i * 2, 2), NumberStyles.HexNumber, null, out writeBytes[size - i - 1]))
                {
                    message = $"入力された16進数文字列が不正です: {writeData}";
                    return false;
                }
            }

            // レジスタライト処理のDLL関数コール
            if (!SUSlv_Reg_Write(WinUsbHandle, address, (uint)size, writeBytes))
            {
                message = $"アドレス 0x{address:X8} の{size * 8}ビットレジスタの書き込みに失敗しました: {writeData}";
                return false;
            }

            message = $"アドレス 0x{address:X8} の{size * 8}ビットレジスタの書き込みに成功しました: {writeData}";
            return true;
        }

        /// <summary>
        /// メモリから可変長のデータを読みだします
        /// </summary>
        /// <param name="dWinUsbHandle">動作を実行する USB デバイスの WinUSB ハンドルを指定します</param>
        /// <param name="FileSize">読み出すバイト数を指定します</param>
        /// <param name="pBuffer">読み出すデータが格納されるバッファのポインタを指定します</param>
        /// <returns>データ読み込みが成功した場合  TRUE を返します。それ以外は FALSE を返します。</returns>
        [DllImport("SUSlv.dll")]
        public static extern bool SUSlv_Data_Read(IntPtr dWinUsbHandle, uint FileSize, IntPtr pBuffer);


        /// <summary>
        /// メモリから可変長のデータを読みだします
        /// </summary>
        /// <param name="path">読み出すデータファイルのパスを指定します</param>
        /// <param name="length">読み出すバイト数を指定します</param>
        /// <param name="message">ステータスメッセージを返します</param>
        /// <param name="time">経過時間をミリ秒で返します</param>
        /// <returns>データ読み込みが成功した場合  TRUE を返します。それ以外は FALSE を返します。</returns>
        public static bool DataRead(string path, int length, out string message, out long time)
        {
            time = -1;

            // 有効なハンドルが取得できていないときはエラー
            if (WinUsbHandle == IntPtr.Zero)
            {
                message = "USBオープンされていません。";
                return false;
            }

            const int UNIT_LENGTH = 4 * 1024;
            if (length % UNIT_LENGTH != 0)
            {
                message = $"リード転送レングスが不正です。4kBの倍数で指定してください: '{length}'";
                return false;
            }

            IntPtr buffer;
            try
            {
                buffer = Marshal.AllocHGlobal(length);
            }
            catch (Exception ex)
            {
                message = $"グローバルヒープの確保に失敗しました: {ex.Message}";
                return false;
            }

            var stopWatch = Stopwatch.StartNew();

            if (!SUSlv_Data_Read(SUS.WinUsbHandle, (uint)length, buffer))
            {
                message = $"メモリからデータの読み出しに失敗しました: {length}バイト";
                return false;
            }

            try
            {
                var bytes = new byte[length];
                Marshal.Copy(buffer, bytes, 0, length);
                using (var stream = new FileStream(path, FileMode.Create, FileAccess.Write))
                using (var writer = new BinaryWriter(stream))
                {
                    writer.Write(bytes);
                }
            }
            catch (IOException ex)
            {
                message = $"ファイルへ書き込めませんでした: {ex.Message}";
                return false;
            }
            catch (Exception ex)
            {
                message = $"読み出しデータのファイル保存に失敗しました: {ex.Message}";
                return false;
            }

            time = stopWatch.ElapsedMilliseconds;

            message = $"メモリからデータの読み出しに成功しました: {length}バイト";
            return true;
        }

        /// <summary>
        /// メモリに可変長のデータを書き込みます。 
        /// </summary>
        /// <param name="dWinUsbHandle">動作を実行する USB デバイスの WinUSB ハンドルを指定します</param>
        /// <param name="FileSize">書き込むバイト数を指定します</param>
        /// <param name="pBuffer">書き込むデータが入ったバッファのポインタを指定します</param>
        /// <returns></returns>
        [DllImport("SUSlv.dll")]
        public static extern bool SUSlv_Data_Write(IntPtr dWinUsbHandle, uint FileSize, IntPtr pBuffer);

        /// <summary>
        /// メモリに可変長のデータを書き込みます。 
        /// </summary>
        /// <param name="path">書き込むデータファイルのパスを指定します</param>
        /// <param name="length">書き込むバイト数を指定します</param>
        /// <param name="message">ステータスメッセージを返します</param>
        /// <param name="time">経過時間をミリ秒で返します</param>
        /// <returns>データ書き込みが成功した場合  TRUE を返します。それ以外は FALSE を返します。</returns>
        public static bool DataWrite(string path, int length, out string message, out long time)
        {
            time = -1;

            if (WinUsbHandle == IntPtr.Zero)
            {
                message = "USBオープンされていません。";
                return false;
            }

            const uint UNIT_LENGTH = 32 * 1024;
            if (length % UNIT_LENGTH != 0)
            {
                message = $"ライト転送レングスが不正です。32kBの倍数で指定してください: '{length}'";
                return false;
            }

            var stopWatch = Stopwatch.StartNew();
            IntPtr buffer;
            try
            {
                var bytes = new byte[length];
                buffer = Marshal.AllocHGlobal(length);
                using (var stream = new FileStream(path, FileMode.Open, FileAccess.Read))
                using (var reader = new BinaryReader(stream))
                {
                    reader.Read(bytes, 0, length);
                }
                Marshal.Copy(bytes, 0, buffer, length);
            }
            catch (IOException ex)
            {
                message = $"ファイルが読み取れませんでした: {ex.Message}";
                return false;
            }
            catch (Exception ex)
            {
                message = $"グローバルヒープの確保に失敗しました: {ex.Message}";
                return false;
            }

            if (!SUSlv_Data_Write(dWinUsbHandle, (uint)length, buffer))
            {
                message = $"メモリからデータの書き込みに失敗しました: {length}バイト";
                return false;
            }

            time = stopWatch.ElapsedMilliseconds;
            message = $"メモリからデータの書き込みに成功しました: {length}バイト";
            return true;
        }

        /// <summary>
        /// FPGA をコンフィグします。（適用ボード：KX-Card7、AX-Card7、SX-Card7、AX-Card7M）
        /// </summary>
        /// <param name="dWinUsbHandle">動作を実行する USB デバイスの WinUSB ハンドルを指定します。</param>
        /// <param name="PathName">FPGA コンフィグファイルを指定します </param>
        /// <param name="Encrypt">暗号化したコンフィグデータを指定する場合、”1”を指定します。 </param>
        /// <returns>FPGA コンフィグが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        [DllImport("SUSlv.dll", CharSet = CharSet.Unicode)]
        public static extern bool SUSlv_Usb_Config_KXCARD7(IntPtr dWinUsbHandle, string PathName, bool Encrypt);

        /// <summary>
        /// FPGA をコンフィグします。（適用ボード：KX-Card7、AX-Card7、SX-Card7、AX-Card7M）
        /// </summary>
        /// <param name="dWinUsbHandle">動作を実行する USB デバイスの WinUSB ハンドルを指定します。</param>
        /// <param name="PathName">FPGA コンフィグファイルを指定します </param>
        /// <param name="message">ステータスメッセージを返します</param>
        /// <param name="time">経過時間をミリ秒で返します</param>
        /// <returns>FPGA コンフィグが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        public static bool Config_KXCARD7(string PathName, bool Encrypt, out string message, out long time)
        {
            // 処理時間を-1で初期化
            time = -1;

            // 有効なハンドルが取得できていないときはエラー
            if (WinUsbHandle == IntPtr.Zero)
            {
                message = "USBオープンされていません。";
                return false;
            }

            var stopWatch = Stopwatch.StartNew();

            // FPGAコンフィグ処理のDLL関数コール
            if (!SUSlv_Usb_Config_KXCARD7(WinUsbHandle, PathName, Encrypt))
            {
                message = $"FPGAコンフィグが失敗しました: {PathName}";
                return false;
            }

            message = $"FPGAコンフィグが成功しました: {PathName}";
            time = stopWatch.ElapsedMilliseconds;
            return true;
        }

        /// <summary>
        /// FPGA をコンフィグします。(適用ボード：CX-Card5) 
        /// </summary>
        /// <param name="dWinUsbHandle">動作を実行する USB デバイスの WinUSB ハンドルを指定します。</param>
        /// <param name="PathName">FPGA コンフィグファイルを指定します </param>
        /// <returns>FPGA コンフィグが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        [DllImport("SUSlv.dll", CharSet = CharSet.Unicode)]
        public static extern bool SUSlv_Usb_Config_CXCARD5(IntPtr dWinUsbHandle, string PathName);

        /// <summary>
        /// FPGA をコンフィグします。(適用ボード：CX-Card5) 
        /// </summary>
        /// <param name="dWinUsbHandle">動作を実行する USB デバイスの WinUSB ハンドルを指定します。</param>
        /// <param name="PathName">FPGA コンフィグファイルを指定します </param>
        /// <param name="message">ステータスメッセージを返します</param>
        /// <param name="time">経過時間をミリ秒で返します</param>
        /// <returns>FPGA コンフィグが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        public static bool Config_CXCARD5(string PathName, out string message, out long time)
        {
            // エラーメッセージ文字列変数を空で初期化
            time = -1;

            // 有効なハンドルが取得できていないときはエラー
            if (WinUsbHandle == IntPtr.Zero)
            {
                message = "USBオープンされていません。";
                return false;
            }

            var stopWatch = Stopwatch.StartNew();

            // FPGAコンフィグ処理のDLL関数コール
            if (!SUSlv_Usb_Config_CXCARD5(WinUsbHandle, PathName))
            {
                message = $"FPGAコンフィグが失敗しました: {PathName}";
                return false;
            }

            message = $"FPGAコンフィグが成功しました: {PathName}";
            time = stopWatch.ElapsedMilliseconds;
            return true;
        }

        /// <summary>
        /// コンフィグ状態の FPGA を未コンフィグ状態（コンフィグする前の状態）にします。
        /// </summary>
        /// <param name="dWinUsbHandle">動作を実行する USB デバイスの WinUSB ハンドルを指定します。 </param>
        /// <returns>FPGA コンフィグが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        [DllImport("SUSlv.dll")]
        public static extern bool SUSlv_Usb_Deconfig(IntPtr dWinUsbHandle);

        /// <summary>
        /// コンフィグ状態の FPGA を未コンフィグ状態（コンフィグする前の状態）にします。
        /// </summary>
        /// <returns>FPGA コンフィグが成功した場合 TRUE を返します。それ以外は FALSE を返します。</returns>
        public static bool Deconfig(out string message)
        {
            // 有効なハンドルが取得できていないときはエラー
            if (WinUsbHandle == IntPtr.Zero)
            {
                message = "USBオープンされていません。";
                return false;
            }

            // 未コンフィグ状態のDLL関数コール
            if (!SUSlv_Usb_Deconfig(WinUsbHandle))
            {
                message = $"未コンフィグ状態への変更が失敗しました。";
                return false;
            }

            message = $"未コンフィグ状態への変更が成功しました。";
            return true;
        }
    }
}
