﻿using System.ComponentModel;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace RefApp8
{
    /// <summary>
    /// TabPageConfig.xaml の相互作用ロジック
    /// </summary>
    public partial class TabPageConfig : UserControl
    {
        /// <summary>
        /// 値未設定を表す文字列です
        /// </summary>
        public const string NONE = "ー";

        /// <summary>
        /// FPGAコンフィグファイルのパスです
        /// </summary>
        public string ConfigDataPath
        {
            get
            {
                return Util.Config.ConfigDataPath;
            }
            set
            {
                if (string.IsNullOrEmpty(value))
                {
                    FilePath.Text = Util.Config.ConfigDataPath = string.Empty;
                    return;
                }
                if (!File.Exists(value))
                {
                    MessageBox.Show($"コンフィグファイルが存在しません。\n{value}",
                        MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                    MainWindow.LogView.Print($"コンフィグファイルが存在しません：{value}");
                    FilePath.Text = string.Empty;
                    return;
                }
                Util.Config.ConfigDataPath = value;
            }
        }

        /// <summary>
        /// FPGAチップベンダーの種別です
        /// </summary>
        public static bool IsVenderXilinx
        {
            get
            {
                return Util.Config.IsVenderXilinx;
            }
            set
            {
                Util.Config.IsVenderXilinx = value;
            }
        }

        public static bool IsVenderAltera
        {
            get
            {
                return Util.Config.IsVenderAltera;
            }
            set
            {
                Util.Config.IsVenderAltera = value;
            }
        }

        /// <summary>
        /// バックグランドワーカーインスタンス
        /// </summary>
        private readonly BackgroundWorker _backgroundWorker;

        /// <summary>
        /// ステータスメッセージです
        /// </summary>
        private string _message;

        /// <summary>
        /// API実行結果です
        /// </summary>
        private bool _result;

        /// <summary>
        /// 処理時間です
        /// </summary>
        private long _time;

        /// <summary>
        /// コンストラクタです
        /// </summary>
        public TabPageConfig()
        {
            InitializeComponent();

            // バックグランドワーカーインスタンスの初期化
            _backgroundWorker = new BackgroundWorker();
            _backgroundWorker.DoWork += StartConfig;
            _backgroundWorker.RunWorkerCompleted += FinishedConfig;
        }

        /// <summary>
        /// [ファイル設定]-[ファイル選択]ボタンのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ButtonSelectFile_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new System.Windows.Forms.OpenFileDialog
            {
                Filter = IsVenderXilinx ? "FPGAコンフィグファイル(*.bin)|*.bin|全て(*.*)|*.*" : "FPGAコンフィグファイル(*.rbf)|*.rbf|全て(*.*)|*.*",
                RestoreDirectory = true,
                CheckFileExists = false,
                CheckPathExists = true
            };
            if (File.Exists(ConfigDataPath))
            {
                dialog.FileName = Path.GetFileName(ConfigDataPath);
                dialog.InitialDirectory = Path.GetDirectoryName(ConfigDataPath);
            }

            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                FilePath.Text = ConfigDataPath = dialog.FileName;
            }
        }

        /// <summary>
        /// [Configure...]ボタンクリック時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ButtonConfigure_Click(object sender, RoutedEventArgs e)
        {
            ConfigStatus.Foreground = ConfigTime.Foreground = Brushes.Black;
            ConfigStatus.Text = ConfigTime.Text = NONE;

            if (!File.Exists(ConfigDataPath))
            {
                MainWindow.LogView.Print($"FPGAコンフィグファイルが存在しません：{ConfigDataPath}");
                MessageBox.Show($"FPGAコンフィグファイルが存在しません。\n{ConfigDataPath}",
                    MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                ConfigStatus.Foreground = Brushes.Red;
                ConfigStatus.Text = "FPGAコンフィグファイルが存在しません。";
                return;
            }

            var cofirmMessage = $"FPGAコンフィグを実行します。よろしいですか？\n{ConfigDataPath}";
            var answer = MessageBox.Show(cofirmMessage, "確認", MessageBoxButton.YesNo, MessageBoxImage.Question);
            if (answer == MessageBoxResult.No)
            {
                return;
            }
            MainWindow.LogView.Print($"FPGAコンフィグの実行中です: {ConfigDataPath}");
            ConfigPanel.IsEnabled = false;
            Mouse.OverrideCursor = Cursors.Wait;

            // FPGAコンフィグ処理をバックグランドで実行
            _backgroundWorker.RunWorkerAsync();
        }

        /// <summary>
        /// FPGAコンフィグ処理のバックグランドワーカースレッド開始時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void StartConfig(object sender, DoWorkEventArgs e)
        {
            if (IsVenderXilinx)
            {
                _result = SUS.Config_KXCARD7(ConfigDataPath, false, out _message, out _time);
            }
            else 
            {
                _result = SUS.Config_CXCARD5(ConfigDataPath, out _message, out _time);
            }

        }

        /// <summary>
        /// FPGAコンフィグ処理のバックグランドワーカースレッド完了時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void FinishedConfig(object sender, RunWorkerCompletedEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Arrow;
            ConfigPanel.IsEnabled = true;
            if (!_result)
            {
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                ConfigStatus.Foreground = ConfigTime.Foreground = Brushes.Red;
                ConfigStatus.Text = _message;
                ConfigTime.Text = NONE;
                return;
            }

            MainWindow.LogView.Print(_message);
            MainWindow.LogView.Print($"→ ファイル名 [{ConfigDataPath}]", OutputStatusBar.OFF);
            MainWindow.LogView.Print($"→ コンフィグ時間: {_time}ms", OutputStatusBar.OFF);
            ConfigStatus.Foreground = ConfigTime.Foreground = Brushes.Blue;
            ConfigStatus.Text = "FPGAコンフィグが成功しました。";
            ConfigTime.Text = $"{_time}ms";
        }

        /// <summary>
        /// [Clear]ボタンクリック時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ButtonDeconfig_Click(object sender, RoutedEventArgs e)
        {
            ConfigStatus.Foreground = ConfigTime.Foreground = Brushes.Black;
            ConfigStatus.Text = ConfigTime.Text = NONE;

            var cofirmMessage = $"FPGAデコンフィグを実行します。よろしいですか？";
            var answer = MessageBox.Show(cofirmMessage, "確認", MessageBoxButton.YesNo, MessageBoxImage.Exclamation);
            if (answer == MessageBoxResult.No)
            {
                return;
            }

            if (!SUS.Deconfig(out string message))
            {
                MainWindow.LogView.Print(message);
                MessageBox.Show(message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                ConfigStatus.Foreground = ConfigTime.Foreground = Brushes.Red;
                ConfigStatus.Text = message;
                ConfigTime.Text = NONE;
                return;
            }

            MainWindow.LogView.Print(message);
            ConfigStatus.Foreground = ConfigTime.Foreground = Brushes.Blue;
            ConfigStatus.Text = message;
            ConfigTime.Text = NONE;
        }

        /// <summary>
        /// FPGAコンフィグファイルのパスをアプリケーション設定ファイルに保存します
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void FilePath_LostFocus(object sender, RoutedEventArgs e)
        {
            ConfigDataPath = FilePath.Text;
        }

        /// <summary>
        /// FPGAベンダー種別選択ラジオボタン変更時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void Vender_Checked(object sender, RoutedEventArgs e)
        {
            if (sender == VenderXilinx)
            {
                IsVenderXilinx = true;
            }
            else
            {
                IsVenderXilinx = false;
            }
        }
    }
}
