﻿using System;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Diagnostics;

namespace RefApp8
{
    /// <summary>
    /// メモリ操作の種類の列挙値です
    /// </summary>
    enum MemoryIO
    {
        Read,
        Write
    }

    /// <summary>
    /// TabPageMemory.xaml の相互作用ロジック
    /// </summary>
    public partial class TabPageMemory : UserControl
    {
        /// <summary>
        /// バックグランドワーカーインスタンス
        /// </summary>
        private readonly BackgroundWorker _backgroundWorker;

        /// <summary>
        /// ステータスメッセージ
        /// </summary>
        private string _message = "";

        /// <summary>
        /// 転送レングス
        /// </summary>
        private int _length = 0;

        /// <summary>
        /// 転送時間
        /// </summary>
        private long _time = -1;

        /// <summary>
        /// API実行結果
        /// </summary>
        private bool _result = false;

        /// <summary>
        /// メモリ読み出し・書き込みデータファイルのパスです
        /// </summary>
        public string MemoryDataPath {
            get
            {
                return Util.Config.MemoryDataPath;
            }
            set
            {
                if (string.IsNullOrEmpty(value))
                {
                    FilePath.Text = Util.Config.MemoryDataPath = string.Empty;
                    return;
                }
                var dir = Path.GetDirectoryName(value);
                if (string.IsNullOrEmpty(dir) || !Directory.Exists(dir))
                {
                    MainWindow.LogView.Print($"フォルダが存在しません：{value}");
                    MessageBox.Show($"フォルダが存在しません。\n{value}", MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                    FilePath.Text = string.Empty;
                    return;
                }
                Util.Config.MemoryDataPath = value;
            }
        }

        /// <summary>
        /// コンストラクタです
        /// </summary>
        public TabPageMemory()
        {
            InitializeComponent();

            // バックグランドワーカーインスタンスの初期化
            _backgroundWorker = new BackgroundWorker();
            _backgroundWorker.DoWork += StartMemoryIO;
            _backgroundWorker.RunWorkerCompleted += FinishedMemoryIO;
        }

        /// <summary>
        /// ターゲットメモリ(レジスタ1)選択時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>

        private void MemorySelectRegister_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // 起動時はイベント発火しない
            if (SUS.WinUsbHandle == IntPtr.Zero)
            {
                return;
            }

            if (!SUS.RegWrite(1, SUS.RegisterWidth.RANGE_8BIT, $"{MemorySelectRegister.SelectedIndex:d2}", out _message))
            {
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            MainWindow.LogView.Print(_message);
        }

        /// <summary>
        /// [メモリアドレスレジスタの設定]-[転送開始アドレス(レジスタ0)]-[Read]ボタンのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ButtonAddressRead_Click(object sender, RoutedEventArgs e)
        {
            if (!SUS.RegRead(0, SUS.RegisterWidth.RANGE_32BIT, out string readData, out _message))
            {
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            ReadMemoryAdress.Text = readData;
            MainWindow.LogView.Print(_message);
        }

        /// <summary>
        /// [メモリアドレスレジスタの設定]-[転送開始アドレス(レジスタ0)]-[Write]ボタンのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ButtonAddressWrite_Click(object sender, RoutedEventArgs e)
        {
            SUS.RegWrite(0, SUS.RegisterWidth.RANGE_32BIT, WriteMemoryAddress.Text, out _message);
            MainWindow.LogView.Print(_message);
        }

        /// <summary>
        /// [ファイル設定]-[ファイル選択]ボタンのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void ButtonFileSelect_Click(object sender, RoutedEventArgs e)
        {
            string _message;
            if (!int.TryParse(Length.Text, NumberStyles.HexNumber, null, out _length))
            {
                _message = $"リード/ライト転送レングスが不正です。16進数文字列で指定してください：'{Length.Text}'";
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            var dialog = new System.Windows.Forms.OpenFileDialog
            {
                Filter = "BINファイル(*.bin)|*.bin|全て(*.*)|*.*",
                RestoreDirectory = true,
                CheckFileExists = false,
                CheckPathExists = true
            };
            if (File.Exists(MemoryDataPath))
            {
                dialog.FileName = Path.GetFileName(MemoryDataPath);
                dialog.InitialDirectory = Path.GetDirectoryName(MemoryDataPath);
            }

            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                FilePath.Text = MemoryDataPath = dialog.FileName;
            }
        }

        /// <summary>
        /// [ファイル設定]-[Memory Read]ボタンのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void MemoryRead_Click(object sender, RoutedEventArgs e)
        {
            string _message;
            if (!int.TryParse(Length.Text, NumberStyles.HexNumber, null, out _length))
            {
                _message = $"リード転送レングスが不正です。16進数文字列で指定してください: '{Length.Text}'";
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            if (FilePath.Text == "")
            {
                _message = $"メモリリードするデータファイルを指定してください。";
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            // メモリリード操作をバックグランドワーカースレッドで実行
            MainWindow.LogView.Print($"メモリリード中です: アドレス[0x{WriteMemoryAddress.Text}] / レングス[0x{Length.Text}]");
            PanelMemoryRead.IsEnabled = false;
            Mouse.OverrideCursor = Cursors.Wait;
            _backgroundWorker.RunWorkerAsync(MemoryIO.Read);
        }

        /// <summary>
        /// [ファイル設定]-[Memory Write]ボタンのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void MemoryWrite_Click(object sender, RoutedEventArgs e)
        {
            string _message;
            if (!int.TryParse(Length.Text, NumberStyles.HexNumber, null, out _length))
            {
                _message = $"ライト転送レングスが不正です。16進数文字列で指定してください: '{Length.Text}'";
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            if (FilePath.Text == "")
            {
                _message = $"メモリライトするデータファイルを指定してください。";
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            if (!File.Exists(FilePath.Text))
            {
                _message = $"メモリライトするデータファイルが存在しません: {FilePath.Text}";
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            // メモリライト操作をバックグランドワーカースレッドで実行
            MainWindow.LogView.Print($"メモリライト中です: アドレス[0x{WriteMemoryAddress.Text}] / レングス[0x{Length.Text}]");
            PanelMemoryRead.IsEnabled = false;
            Mouse.OverrideCursor = Cursors.Wait;
            _backgroundWorker.RunWorkerAsync(MemoryIO.Write);
        }

        /// <summary>
        /// メモリ読み出し・書き込みデータファイルのパスをアプリケーション設定ファイルに保存します
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void FilePath_LostFocus(object sender, RoutedEventArgs e)
        {
            MemoryDataPath = FilePath.Text;
        }


        /// <summary>
        /// メモリ入出力操作のワーカースレッド処理です
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void StartMemoryIO(object sender, DoWorkEventArgs e)
        {
            var io = (MemoryIO)e.Argument;
            if (io == MemoryIO.Read)
            {
                _result = SUS.DataRead(MemoryDataPath, _length, out _message, out _time);
            }
            else
            {
                _result = SUS.DataWrite(MemoryDataPath, _length, out _message, out _time);
            }
        }

        /// <summary>
        /// メモリ入出力操作完了時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void FinishedMemoryIO(object sender, RunWorkerCompletedEventArgs e)
        {
            Mouse.OverrideCursor = Cursors.Arrow;

            if (!_result)
            {
                MainWindow.LogView.Print(_message);
                MessageBox.Show(_message, MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
                PanelMemoryRead.IsEnabled = true;
                return;
            }

            var transRate = _length / _time;
            MainWindow.LogView.Print(_message);
            MainWindow.LogView.Print($"→ ファイル名 [{FilePath.Text}]", OutputStatusBar.OFF);
            MainWindow.LogView.Print($"→ 転送時間: {_time}ms, 転送レート: {transRate}KBytes/s, 転送バイト数: {_length}Bytes", OutputStatusBar.OFF);
            var message = $"{_message}\n→ 転送時間: {_time}ms, 転送レート: {transRate}KBytes/s";
            MessageBox.Show(message, "メモリ操作");
            PanelMemoryRead.IsEnabled = true;
        }

        /// <summary>
        /// バイナリデータを外部アプリで表示します
        /// ここでは、使用するアプリのフルパスを指定しています。
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void AppOpen_Click(object sender, EventArgs e)
        {
            ProcessStartInfo pInfo = new ProcessStartInfo();
            pInfo.Arguments = "true";
            pInfo.FileName = @"D:\\temp\\program\\FavBinEdit-1-2-5\\FavBinEdit.exe";
            pInfo.Arguments = @MemoryDataPath;

        
            Process.Start(pInfo);

        }
    }
}
