﻿using System;
using System.Configuration;
using System.IO;

namespace RefApp8.Util
{
    /// <summary>
    /// アプリケーション設定ファイルのユーティリティクラスです
    /// </summary>
    internal static class Config
    {
        /// <summary>
        /// 構成ファイルクラスインスタンスです
        /// </summary>
        private static Configuration configuration;

        /// <summary>
        /// 指定したクライアント構成ファイルを Configuration オブジェクトとして開きます
        /// </summary>
        public static void Open()
        {
            string appName = Path.Combine(AppData.ConfigDirectory, "RefApp8");
            string configPath = appName + ".config";
            if (!File.Exists(configPath))
            {
                using (var writer = new StreamWriter(configPath))
                {
                    writer.WriteLine("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\r\n<configuration>\r\n</configuration>");
                }
            }
            var exeFileMap = new ExeConfigurationFileMap { ExeConfigFilename = configPath };
            configuration = ConfigurationManager.OpenMappedExeConfiguration(exeFileMap, ConfigurationUserLevel.None);
        }

        /// <summary>
        /// クライアント構成ファイルから指定されたキーの値を返します
        /// </summary>
        /// <param name="key">クライアント構成ファイルのキー</param>
        /// <param name="defaultValue">デフォルト値</param>
        /// <returns>指定されたキーの値</returns>
        public static string GetValue(string key, string defaultValue)
        {
            var settings = configuration.AppSettings.Settings;
            if (settings[key] == null)
            {
                SetValue(key, defaultValue);
            }
            return settings[key].Value;
        }

        /// <summary>
        /// クライアント構成ファイルに指定されたキーの値を保存します
        /// </summary>
        /// <param name="key">クライアント構成ファイルのキー</param>
        /// <param name="value">更新する値</param>
        public static void SetValue(string key, string value)
        {
            var settings = configuration.AppSettings.Settings;
            if (settings[key] == null)
            {
                settings.Add(key,value);
                configuration.Save(ConfigurationSaveMode.Full);
            }
            else if (settings[key].Value != value)
            {
                settings[key].Value = value;
                configuration.Save(ConfigurationSaveMode.Full);
            }
        }

        /// <summary>
        /// メモリ読み出し・書き込みデータファイルのパスです
        /// </summary>
        public static string MemoryDataPath
        {
            get
            {
                var defaultaDataDir = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                var defaultDataPath = Path.Combine(defaultaDataDir, "MEMORY.bin");
                var dataPath = GetValue("MemoryDataPath", defaultDataPath);
                return dataPath;
            }
            set
            {
                SetValue("MemoryDataPath", value);
            }
        }

        /// <summary>
        /// FPGAコンフィグファイルのパスです
        /// </summary>
        public static string ConfigDataPath
        {
            get
            {
                var defaultaDataDir = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                var defaultDataPath = Path.Combine(defaultaDataDir, "Config.bin");
                var dataPath = GetValue("ConfigDataPath", defaultDataPath);
                return dataPath;
            }
            set
            {
                SetValue("ConfigDataPath", value);
            }
        }

        /// <summary>
        /// FPGAチップベンダーの種別です
        /// </summary>
        public static bool IsVenderXilinx
        {
            get
            {
                var isVenderXilinx = GetValue("IsVenderXilinx", "1");
                return isVenderXilinx == "1";
            }
            set
            {
                SetValue("IsVenderXilinx", value ? "1" : "0");
            }
        }

        public static bool IsVenderAltera
        {
            get
            {
                var isVenderAltera = GetValue("IsVenderAltera", "1");
                return isVenderAltera == "1";
            }
            set
            {
                SetValue("IsVenderAltera", value ? "1" : "0");
            }
        }
    }
}
